#include "common.h"

#include <appext.h>
#include "resource.h"

extern UINT g_cRefDLL;
extern HINSTANCE g_hinstDLL;

const CHAR c_aszVerb[] = "Task2Mail Plugin";  // not UNICODE!
UINT g_idCmd = 0;


static
HRESULT
IsTaskCompleted(
	CEOID			oid,
	VARIANT_BOOL*	pvtbCompleted
)
{
	HRESULT			hr = E_FAIL;
	IPOutlookApp	*polApp		= NULL;
	ITask			*pTask		= NULL;
	BSTR			pwszSubject	= NULL;
	BSTR			pwszBody	= NULL;
	VARIANT_BOOL	vtbCompleted = VARIANT_TRUE;

	hr = CoInitializeEx(NULL, COINIT_MULTITHREADED);

	if (SUCCEEDED(hr)) {
		hr = CoCreateInstance(CLSID_Application, NULL, CLSCTX_INPROC_SERVER, IID_IPOutlookApp, (LPVOID *)&polApp);

		if (SUCCEEDED(hr)) {

			hr = polApp->Logon(NULL);

			if (SUCCEEDED(hr)) {
				hr = polApp->GetItemFromOid(oid, (IDispatch**)&pTask);

				if (SUCCEEDED(hr)) {
					hr = pTask->get_Complete(&vtbCompleted);

					if (SUCCEEDED(hr)) {
						*pvtbCompleted = vtbCompleted;
					}

					pTask->Release();
				}

				polApp->Logoff();
			}

			polApp->Release();
		}

		CoUninitialize();
	}

	return hr;
}

static
HRESULT
ToggleTaskCompleted(
	CEOID	oid
)
{
	HRESULT			hr = E_FAIL;
	IPOutlookApp	*polApp		= NULL;
	ITask			*pTask		= NULL;
	VARIANT_BOOL	vtbCompleted = VARIANT_FALSE;

	hr = CoInitializeEx(NULL, COINIT_MULTITHREADED);

	if (SUCCEEDED(hr)) {
		hr = CoCreateInstance(CLSID_Application, NULL, CLSCTX_INPROC_SERVER, IID_IPOutlookApp, (LPVOID *)&polApp);

		if (SUCCEEDED(hr)) {
			hr = polApp->Logon(NULL);

			if (SUCCEEDED(hr)) {
				hr = polApp->GetItemFromOid(oid, (IDispatch**)&pTask);

				if (SUCCEEDED(hr)) {
					hr = pTask->get_Complete(&vtbCompleted);

					if (SUCCEEDED(hr)) {
						hr = pTask->put_Complete((vtbCompleted == VARIANT_TRUE) ? VARIANT_FALSE : VARIANT_TRUE);

						if (SUCCEEDED(hr)) {
							hr = pTask->Save();
						}
					}

					pTask->Release();
				}

				polApp->Logoff();
			}

			polApp->Release();
		}

		CoUninitialize();
	}

	return hr;
}

static
HRESULT
GetOIDFromIDO(
	IDataObject*	pIdo,
	CEOID*			pOid
)
{
	FORMATETC fmt;
	STGMEDIUM stg;
	HRESULT	hr = E_FAIL;

	if (pIdo == NULL || pOid == NULL) {
		return MAKE_HRESULT(SEVERITY_ERROR, FACILITY_STORAGE, ERROR_INVALID_PARAMETER);
	}

	ZeroMemory(&fmt, sizeof(fmt));
	fmt.cfFormat = ::RegisterClipboardFormat(CFNAME_ITEMREFARRAY);
	fmt.dwAspect = DVASPECT_CONTENT;
	fmt.lindex = -1;
	fmt.ptd = NULL;
	fmt.tymed = TYMED_HGLOBAL;

	ZeroMemory(&stg, sizeof(stg));
	hr = pIdo->GetData(&fmt, &stg);
	if (FAILED(hr)) {
		hr = MAKE_HRESULT(SEVERITY_ERROR, FACILITY_STORAGE, ERROR_INVALID_PARAMETER);
	}
	else {
		if (stg.hGlobal != NULL) {
			ItemRefArray*	pItemRefArray	= NULL;
			pItemRefArray = (ItemRefArray *)stg.hGlobal;

			if (pItemRefArray != NULL) {
				*pOid = (CEOID)pItemRefArray->rgRefs[0].pRef;
			}
			else {
				hr = MAKE_HRESULT(SEVERITY_ERROR, FACILITY_STORAGE, ERROR_INVALID_PARAMETER);
			}
		}
		else {
			hr = MAKE_HRESULT(SEVERITY_ERROR, FACILITY_STORAGE, ERROR_INVALID_PARAMETER);
		}
	}

	return hr;
}


//****************************************************************************
// Function Name: CTasksExt
// 
// Purpose: constructor for the class
// 
// Arguments:
//    NONE
//
// Return Values:
//    NONE
//
// Side Effects:
//
// Description:
//
//****************************************************************************
CTasksExt::CTasksExt()
{
	m_cRef = 0;
	m_punkSite = NULL;
	++g_cRefDLL;
}


//****************************************************************************
// Function Name: ~CTasksExt
// 
// Purpose: destructor for the class
// 
// Arguments:
//    NONE
//
// Return Values:
//    NONE
// 
// Side Effects:
//
// Description:
//
//****************************************************************************
CTasksExt::~CTasksExt()
{	
	ASSERT(0 == m_cRef);

	if (m_punkSite != NULL) {
		m_punkSite->Release();
	}
    
	--g_cRefDLL;
}


//****************************************************************************
// Function Name: AddRef
//
// Purpose:  IUnknown::AddRef
//
// Arguments:
//    NONE
//
// Return Values:
//    ULONG - new reference count
//
// Side Effects:
//
// Description:
//
//****************************************************************************
STDMETHODIMP_(ULONG) CTasksExt::AddRef()
{
	return ++m_cRef;
}


//****************************************************************************
// Function Name: Release
// 
// Purpose: IUnknown::Release()
// 
// Arguments:
//    NONE
// 
// Return Values:
//    ULONG - new reference count 
//  
// Side Effects: 
//  
// Description: 
//    
//****************************************************************************
STDMETHODIMP_(ULONG)
CTasksExt::Release()
{
	if (--m_cRef == 0) {
		delete this;
		return 0;
	}

	return m_cRef;
}


//****************************************************************************
// Function Name: QueryInterface
// 
// Purpose: IUnknown::QueryInterface
// 
// Arguments:
//    IN REFIID riid - interface identifier
//    OUT LPVOID *ppv - interface pointer returned if supported
//  
// Return Values:
//    HRESULT - success/failure
// 
// Side Effects: 
//  
// Description: 
//    
//****************************************************************************
STDMETHODIMP
CTasksExt::QueryInterface(
	REFIID	riid, 
	LPVOID	*ppv
)
{
	if (NULL == ppv) {
		return E_POINTER;
	}

	*ppv = NULL;

	if (IID_IUnknown == riid) {
		*ppv = (LPVOID)this;
	}
	else if (IID_IContextMenu == riid) {
		*ppv = dynamic_cast<IContextMenu*>(this);
	}
	else if (IID_IObjectWithSite == riid) {
		*ppv = dynamic_cast<IObjectWithSite*>(this);
	}

    if (NULL != *ppv) {
        ((IUnknown *)(*ppv))->AddRef();
		return S_OK;
	}

	return E_NOINTERFACE;
}


//****************************************************************************
// Function Name: GetCommandString
//
// Purpose: get the command string for this context menu item
//
// Arguments:
//    IN UINT idCmd - the menu command identifier
//    IN UINT uFlags - flags specifying what information is being requested
//    IN/OUT UINT *pwReserved - ignored
//    OUT LPSTR pszName - output buffer
//    IN UINT cchMax - length of output buffer
//
// Return Values:
//    HRESULT - SUCCESS/FAILURE
//
// Side Effects:
//
// Description: 
//
//****************************************************************************
STDMETHODIMP CTasksExt::GetCommandString(
	UINT idCmd, 
	UINT uFlags,
    UINT *pwReserved, 
	LPSTR pszName, 
	UINT cchMax
	)
{
	HRESULT hr = NOERROR;

	UNREFERENCED_PARAMETER(pwReserved);

	switch (uFlags) {
		case GCS_HELPTEXT:
			// no help text
			if ((NULL != pszName) && (cchMax > 0)) {
				*pszName = '\0';
			}
			break;

		case GCS_VALIDATE:
            // only one command - does it match?
			if ((0 == g_idCmd) || (idCmd != g_idCmd)) {
				hr = S_FALSE;
			}
			break;

        case GCS_VERB:
			// only one verb
			if (idCmd != g_idCmd) {
				hr = E_FAIL;
			}
			else {
				// copy the verb
				if (NULL != pszName) {
					strncpy(pszName, c_aszVerb, cchMax);
				}
			}
			break;

		default:
			hr = E_NOTIMPL;
			break;
	}

	return hr;

}


//****************************************************************************
// Function Name: InvokeCommand
//
// Purpose: invoke a command from the context menu
//
// Arguments:
//    IN/OUT LPCMINVOKECOMMANDINFO pici - ptr to command info struct
//
// Return Values:
//    HRESULT - SUCCESS/FAILURE
//
// Side Effects:
//
// Description: 
//
//****************************************************************************
STDMETHODIMP
CTasksExt::InvokeCommand(
	LPCMINVOKECOMMANDINFO pici
)
{
	HRESULT hr = NOERROR;

	if (NULL == pici) {
		hr = E_POINTER;
	}
	else {
		if (sizeof(CMINVOKECOMMANDINFO) != pici->cbSize) {
			// there's no CMINVOKECOMMANDINFOEX in WinCE
			hr = E_INVALIDARG;
		}
		else {

			IDataObject *pIdo;
            
			if (FAILED(GetSite(IID_IDataObject, (void **)&pIdo))) {
				hr = MAKE_HRESULT(SEVERITY_ERROR, FACILITY_NULL,  ERROR_NO_SUCH_SITE);
			}
			else {
				CEOID	oid;

				hr = GetOIDFromIDO(pIdo, &oid);
				if (SUCCEEDED(hr)) {
					hr = ToggleTaskCompleted(oid);
				}

				pIdo->Release();
			}
		}
	}

	return hr;

}


//****************************************************************************
// Function Name: QueryContextMenu
//
// Purpose: add commands to the context menu
//
// Arguments:
//    IN HMENU hmenu - handle to the menu
//    IN UINT indexMenu - zero-based position at which to insert the first item
//    IN UINT idCmdFirst - first available command ID
//    IN UINT idCmdLast - last available command ID
//    IN UINT uFlags - optional flags
//
// Return Values:
//    HRESULT - SUCCESS/FAILURE
//
// Side Effects:
//
// Description: 
//
//****************************************************************************
STDMETHODIMP
CTasksExt::QueryContextMenu(
	HMENU hmenu,
	UINT indexMenu, 
	UINT idCmdFirst,
	UINT idCmdLast,
	UINT uFlags
)
{
	HRESULT hr = NOERROR;

	UNREFERENCED_PARAMETER(uFlags);

	if (idCmdLast < idCmdFirst) {
		hr = E_INVALIDARG;
	}
	else {
		IDataObject *pIdo;

		// get the site
		if (FAILED(GetSite(IID_IDataObject, (void **)&pIdo))) {
			hr = MAKE_HRESULT(SEVERITY_ERROR, FACILITY_NULL, ERROR_NO_SUCH_SITE);
		}
		else {
			CEOID oid;

			hr = GetOIDFromIDO(pIdo, &oid);
			if (SUCCEEDED(hr)) {
				VARIANT_BOOL vtbCompleted = VARIANT_FALSE;

				hr =  IsTaskCompleted(oid, &vtbCompleted);
				if (SUCCEEDED(hr)) {
					TCHAR szCommand[64];

					memset(szCommand, 0, sizeof(szCommand));

					if (vtbCompleted == VARIANT_TRUE) {
						LoadString(g_hinstDLL, IDS_WORKING, szCommand, sizeof(szCommand)/sizeof(szCommand[0]));
					}
					else {
						LoadString(g_hinstDLL, IDS_FINISH, szCommand, sizeof(szCommand)/sizeof(szCommand[0]));
					}

					g_idCmd = idCmdFirst;
					if (InsertMenu(hmenu, indexMenu, MF_BYPOSITION, g_idCmd, szCommand)) {
						hr = MAKE_HRESULT(SEVERITY_SUCCESS, 0, 1);
					}
				}
			}

		}
	}

	return hr;
}


//****************************************************************************
// Function Name: GetSite
//
// Purpose: get the owner (site) of the object
//
// Arguments:
//    IN REFIID riid - interface ID to retrieve
//    OUT void **ppvSite - interface pointer
//
// Return Values:
//    HRESULT - SUCCESS/FAILURE
//
// Side Effects:
//
// Description: 
//
//****************************************************************************
STDMETHODIMP
CTasksExt::GetSite(
	REFIID riid, 
	void **ppvSite
)
{
	HRESULT hr = NOERROR;

	if (NULL == ppvSite) {
		hr = E_POINTER;
	}
	else {
		if (NULL == m_punkSite) {
			// no site
			*ppvSite = NULL;
			hr = E_FAIL;
		}
		else {
			// see if we have the requested interface
			hr = m_punkSite->QueryInterface(riid, ppvSite);
		}
	}

	return hr;
}


//****************************************************************************
// Function Name: SetSite
//
// Purpose: set the owner (site) of the object
//
// Arguments:
//    IN IUnknown *pUnkSite - object pointer
//
// Return Values:
//    HRESULT - SUCCESS/FAILURE
//
// Side Effects:
//
// Description: 
//
//****************************************************************************
STDMETHODIMP
CTasksExt::SetSite(
	IUnknown *pUnkSite
)
{
	if (NULL != pUnkSite) {
		pUnkSite->AddRef();
	}

	if (NULL != m_punkSite) {
		m_punkSite->Release();
	}
    
	m_punkSite = pUnkSite;

	return NOERROR;
}
