#include "common.h"

extern UINT g_cRefDLL;

CTasksExtClassFactory::CTasksExtClassFactory()
{
    m_cRef = 0;

    ++g_cRefDLL;
}


CTasksExtClassFactory::~CTasksExtClassFactory()
{
    ASSERT(m_cRef == 0);

    --g_cRefDLL;
}


STDMETHODIMP_(ULONG)
CTasksExtClassFactory::AddRef()
{
    return ++m_cRef;
}


STDMETHODIMP_(ULONG)
CTasksExtClassFactory::Release()
{
    if (--m_cRef == 0) {
        delete this;
        return 0;
    }

    return m_cRef;
}


STDMETHODIMP
CTasksExtClassFactory::QueryInterface(
	REFIID riid,
    LPVOID *ppv
	)
{
    if (ppv == NULL) {
        return E_INVALIDARG;
	}

	*ppv = NULL;

	if (riid == IID_IUnknown) {
		*ppv = (LPVOID)this;
	}
	else if (riid == IID_IClassFactory) {
		*ppv = (IClassFactory *)this;
	}

	if (*ppv != NULL) {
		((IUnknown *)(*ppv))->AddRef();
		return S_OK;
	}

	return E_NOINTERFACE;
}


STDMETHODIMP
CTasksExtClassFactory::CreateInstance(
	LPUNKNOWN	punkOuter,
	REFIID		riid,
	LPVOID*		ppvObj
)
{
	CTasksExt *pobjNew;

	if (ppvObj == NULL) {
		return E_INVALIDARG;
	}

	if (punkOuter != NULL) {
		return CLASS_E_NOAGGREGATION;
	}

	pobjNew = new CTasksExt;
	if (pobjNew == NULL) {
		return E_OUTOFMEMORY;
	}

	return pobjNew->QueryInterface(riid, ppvObj);
}


STDMETHODIMP
CTasksExtClassFactory::LockServer(
	BOOL fLock
)
{
	UNREFERENCED_PARAMETER(fLock);

	return S_OK;
}
